package com.ruoyi.web.controller.zdsz;

import java.util.List;
import java.util.Arrays;
import java.util.concurrent.TimeUnit;

import lombok.RequiredArgsConstructor;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.*;
import cn.dev33.satoken.annotation.SaCheckPermission;
import org.springframework.web.bind.annotation.*;
import org.springframework.validation.annotation.Validated;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.PageQuery;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.zdsz.domain.vo.ZEngineeringInfoVo;
import com.ruoyi.zdsz.domain.bo.ZEngineeringInfoBo;
import com.ruoyi.zdsz.service.IZEngineeringInfoService;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 工程详情
 *
 * @author ruoyi
 * @date 2023-12-27
 */
@Validated
@RequiredArgsConstructor
@RestController
@RequestMapping("/zdsz/engineeringInfo")
public class ZEngineeringInfoController extends BaseController {

    private final IZEngineeringInfoService iZEngineeringInfoService;

    /**
     * 查询工程详情列表
     */
//    @SaCheckPermission("zdsz:engineeringInfo:list")
    @GetMapping("/list")
    public TableDataInfo<ZEngineeringInfoVo> list(ZEngineeringInfoBo bo, PageQuery pageQuery) {
        return iZEngineeringInfoService.queryPageList(bo, pageQuery);
    }

    /**
     * 导出工程详情列表
     */
//    @SaCheckPermission("zdsz:engineeringInfo:export")
    @Log(title = "工程详情", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(ZEngineeringInfoBo bo, HttpServletResponse response) {
        List<ZEngineeringInfoVo> list = iZEngineeringInfoService.queryList(bo);
        ExcelUtil.exportExcel(list, "工程详情", ZEngineeringInfoVo.class, response);
    }

    /**
     * 获取工程详情详细信息
     *
     * @param id 主键
     */
//    @SaCheckPermission("zdsz:engineeringInfo:query")
    @GetMapping("/{id}")
    public R<ZEngineeringInfoVo> getInfo(@NotNull(message = "主键不能为空")
                                     @PathVariable String id) {
        return R.ok(iZEngineeringInfoService.queryById(id));
    }

    /**
     * 新增工程详情
     */
//    @SaCheckPermission("zdsz:engineeringInfo:add")
    @Log(title = "工程详情", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Void> add(@Validated(AddGroup.class) @RequestBody ZEngineeringInfoBo bo) {
        return toAjax(iZEngineeringInfoService.insertByBo(bo));
    }

    /**
     * 修改工程详情
     */
//    @SaCheckPermission("zdsz:engineeringInfo:edit")
    @Log(title = "工程详情", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody ZEngineeringInfoBo bo) {
        return toAjax(iZEngineeringInfoService.updateByBo(bo));
    }

    /**
     * 删除工程详情
     *
     * @param ids 主键串
     */
//    @SaCheckPermission("zdsz:engineeringInfo:remove")
    @Log(title = "工程详情", businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                          @PathVariable String[] ids) {
        return toAjax(iZEngineeringInfoService.deleteWithValidByIds(Arrays.asList(ids), true));
    }
//    @SaCheckPermission("zdsz:engineeringInfo:remove")
    @Log(title = "信息删除", businessType = BusinessType.DELETE)
    @DeleteMapping("/deleteById")
    public R deleteById(@RequestBody  ZEngineeringInfoBo bo) {
        return R.ok(iZEngineeringInfoService.deleteById(bo));
    }

    @Log(title = "施工信息删除", businessType = BusinessType.DELETE)
    @DeleteMapping("/deleteBySZGY")
    public R deleteBySZGY(@RequestBody  ZEngineeringInfoBo bo) {
        return R.ok(iZEngineeringInfoService.deleteBySZGY(bo));
    }

}
